const path = require( 'path' );
const defaultConfig = require( '@wordpress/scripts/config/webpack.config' );
const WooCommerceDependencyExtractionWebpackPlugin = require( '@woocommerce/dependency-extraction-webpack-plugin' );
const { BundleAnalyzerPlugin } = require('webpack-bundle-analyzer');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const defaultRules = defaultConfig.module.rules.filter((rule) => {
	return String(rule.test) !== String(/\.(sc|sa)ss$/);
});

module.exports = {
	...defaultConfig,
	entry: {
		index: path.resolve(
			process.cwd(),
			'block',
			'assets',
			'js',
			'index.js'
		),
		'contact-info-section': path.resolve(
			process.cwd(),
			'block',
			'assets',
			'js',
			'contact-info-section',
			'index.js'
		),
        'contact-info-section-frontend': path.resolve(
            process.cwd(),
            'block',
            'assets',
            'js',
            'contact-info-section',
            'frontend.js'
        ),
        'additional-sections': path.resolve(
            process.cwd(),
            'block',
            'assets',
            'js',
            'additional-sections',
            'index.js'
        ),
        'additional-sections-frontend': path.resolve(
            process.cwd(),
            'block',
            'assets',
            'js',
            'additional-sections',
            'frontend.js'
        ),
	},
	output: {
		filename: '[name].js',
		path: path.resolve( process.cwd(), 'block', 'assets', 'dist' ),
	},
	module: {
        ...defaultConfig.module,
        rules: [
            // SCSS handling rule
            ...defaultRules,
            {
              test: /\.scss$/,
              exclude: /node_modules/,
              use: [
                MiniCssExtractPlugin.loader,
                'css-loader',
                'postcss-loader',
                'sass-loader',
              ],
              include: path.resolve(process.cwd(), 'block', 'assets', 'js'),
            },
        ],
    },
	plugins: [
        //new BundleAnalyzerPlugin(),
        new MiniCssExtractPlugin({
            filename: '[name].css',
        }),
        ...defaultConfig.plugins.filter(
            ( plugin ) =>
                plugin.constructor.name !== 'DependencyExtractionWebpackPlugin'
        ),
        new WooCommerceDependencyExtractionWebpackPlugin(),
    ],
};
